import { CommandInteraction, Guild, GuildMember, PermissionFlags, PermissionResolvable, TextChannel } from "discord.js";
import App from "../App";
import Common from "../functions/Common";
import { Response } from "../types/Types";

// @Exclude-API
export default abstract class SimpleCommand {

    public name: string;
    public description: string;
    public options: [];

    private _member: GuildMember | null;
    private _guild: Guild | null;
    private _channel: TextChannel | null;
    private interaction: CommandInteraction | null;


    constructor(name: string, description: string, options?: []) {
        this.name = name;
        this.description = description;
        this.options = options ?? [];
        this._member = null;
        this._channel = null;
        this._guild = null;
        this.interaction = null;
    }


    async init(member: GuildMember, guild: Guild, channel: TextChannel, interaction: CommandInteraction) {
        this._member = member;
        this._guild = guild;
        this._channel = channel;
        this.interaction = interaction;
        return true;
    }

    abstract execute(app?: App, interaction?: CommandInteraction): Promise<any>;

    get member() {
        return this._member as GuildMember;
    }

    get channel() {
        return this._channel as TextChannel;
    }

    get guild() {
        return this._guild as Guild;
    }

    checkPermission(permission: PermissionResolvable) {
        return this.member.permissions.has(permission);
    }

    checkRole(roleReference: string) {
        return this.member.roles.cache.find(r => r.id == roleReference || r.name == roleReference);
    }

    checkRoles(roleReferences: string[]) {
        return roleReferences.some(r => this.member.roles.cache.find(a => a.name == r || a.id == r));
    }

    isOwner() {
        return this.guild.ownerId == this.member.id;
    }

    getChannelType() {
        return this.channel.type;
    }

    reply(response: Response) {
        return Common.reply(this.interaction, response);
    } 
}