import { ButtonInteraction, CategoryChannel, CommandInteraction, Guild, MessageEmbed, MessageEmbedOptions } from "discord.js";
import {Response} from "../types/Types";

export default class Common {

    private static default: MessageEmbed;

    static setDefaultEmbed(defaultEmbed: MessageEmbed) {
        this.default = defaultEmbed;
    }

    static generateEmbed(newEmbedOptions: MessageEmbedOptions) {
        return this.concatEmbeds(this.default, new MessageEmbed(newEmbedOptions))
    }

    static concatEmbeds(def: MessageEmbed, second: MessageEmbed) {
        if (this.isNull(def.title) && !this.isNull(second.title)) {
            def.title = second.title;
        }

        if (this.isNull(def.description) && !this.isNull(second.description)) {
            def.description = second.description;
        }

        if (this.isNull(def.fields) && !this.isNull(second.fields)) {
            def.fields = second.fields;
        }

        if (this.isNull(def.color) && !this.isNull(second.color)) {
            def.color = second.color;
        }

        if (this.isNull(def.author) && !this.isNull(second.author)) {
            def.author = second.author;
        }

        if (this.isNull(def.footer) && !this.isNull(second.footer)) {
            def.footer = second.footer;
        }

        if (this.isNull(def.thumbnail) && !this.isNull(second.thumbnail)) {
            def.thumbnail = second.thumbnail;
        }

        return def;
    }

    static isNull(val) {
        return val == null || !val || val == "" || val == undefined || val == [] || val == {};
    }

    static async getChannel(guild: Guild, channelReference: string) {
        let channel = guild.channels.cache.find(r => r.name == channelReference || r.id == channelReference);

        if (!channel) {
            guild.channels.fetch(channelReference).then(c => {
                return c || null;
            }).catch(err => {
                return null;
            })
        } else {
            return channel || null;
        }
    }

    static async getMember(guild: Guild, memberReference: string) {
        let member = guild.members.cache.get(memberReference);

        if (!member) {
            guild.members.fetch(memberReference).then(c => {
                return c || null;
            }).catch(err => {
                return null;
            })
        } else {
            return member || null;
        }
    }

    static async getCategory(guild: Guild, id: string, create: boolean) {
        const category: CategoryChannel = (await this.getChannel(guild, id)) as CategoryChannel;
        if (!this.isNull(category) && category.children.size < 49) {
            return {
                isNew: false,
                category: category
            };
        }

        if (create) {
            const channel = await guild.channels.create(category.name, {
                type: "GUILD_CATEGORY"
            });
            return {
                isNew: true,
                category: channel
            };
        } else return {
            isNew: false,
            category: category
        };
    }

    static reply(interaction: CommandInteraction | ButtonInteraction, response: Response) {
        if (response.type == "MESSAGE") {
            if (response.content == null) {
                return interaction?.reply("<SimpleCommand>.reply -> Content property is empty (null)")
            }
            return interaction?.reply(response.content);
        }

        if (response.embeds == null || response.embeds.length == 0) {
            return interaction?.reply("<SimpleCommand>.reply -> Embeds array is empty (null)")
        }

        return interaction?.reply({
            embeds: response.embeds
        })
    }
}